@@ -31,9 +31,14 @@ class Auth:
 
 	:type requests: :class:`~requests.Session`
 	:param requests: Session to make HTTP requests
+
+	:type client_secret: str or dict
+	:param client_secret: (Optional) File path to or the dict object
+		from social client secret file, defaults to :data:`None`.
+
 	"""
 
-	def __init__(self, api_key, credentials, requests):
+	def __init__(self, api_key, credentials, requests, client_secret=None):
 		""" Constructor method """
 
 		self.api_key = api_key
@@ -41,6 +46,60 @@ class Auth:
 		self.requests = requests
 
 		self.current_user = None
+		self.provider_id = None
+		self.session_id = None
+
+		if client_secret:
+			self.client_secret = _load_client_secret(client_secret)
+
+	def create_authentication_uri(self, provider_id):
+		""" Creates an authentication URI for the given social
+		provider.
+
+		| For more details:
+		| |section-fetch-providers-for-email|_
+
+		.. |section-fetch-providers-for-email| replace::
+			Firebase Auth REST API | Fetch providers for email
+
+		.. _section-fetch-providers-for-email:
+			https://firebase.google.com/docs/reference/rest/auth#section-fetch-providers-for-email
+
+
+		:type provider_id: str
+		:param provider_id: The IdP ID. For white listed IdPs it's a
+			short domain name e.g. 'google.com', 'aol.com', 'live.net'
+			and 'yahoo.com'. For other OpenID IdPs it's the OP
+			identifier.
+
+
+		:return: The URI used by the IDP to authenticate the user.
+		:rtype: str
+		"""
+
+		request_ref = "https://www.googleapis.com/identitytoolkit/v3/relyingparty/createAuthUri?key={0}".format(self.api_key)
+
+		data = {
+			"authFlowType": 'CODE_FLOW',
+			"clientId": self.client_secret['client_id'],
+			"providerId": provider_id,
+			"continueUri": self.client_secret['redirect_uris'][0],
+			"customParameter": {
+				"access_type": 'offline',
+				"prompt": 'select_account',
+				"include_granted_scopes": 'true',
+			}
+		}
+
+		headers = {"content-type": "application/json; charset=UTF-8"}
+		request_object = self.requests.post(request_ref, headers=headers, json=data)
+
+		raise_detailed_error(request_object)
+
+		self.provider_id = provider_id
+		self.session_id = request_object.json()['sessionId']
+
+		return request_object.json()['authUri']
 
 	def sign_in_with_email_and_password(self, email, password):
 		""" Sign in a user with an email and password.
@@ -353,6 +412,47 @@ class Auth:
 
 		return request_object.json()
 
+	def sign_in_with_oauth_credential(self, token):
+		""" Sign In With OAuth credential.
+
+		| For more details:
+		| |section-sign-in-with-oauth-credential|_
+
+		.. |section-sign-in-with-oauth-credential| replace::
+			Firebase Auth REST API | Sign in with OAuth credential
+
+		.. _section-sign-in-with-oauth-credential:
+			https://firebase.google.com/docs/reference/rest/auth#section-sign-in-with-oauth-credential
+
+
+		:type token: dict
+		:param token: The OAuth credential (an ID token or access 
+			token).
+
+		:return: User account info and Firebase Auth Tokens.
+		:rtype: dict
+		"""
+
+		request_ref = "https://www.googleapis.com/identitytoolkit/v3/relyingparty/verifyAssertion?key={0}".format(self.api_key)
+
+		data = {
+			'postBody': 'providerId={0}&{1}={2}'.format(self.provider_id, token['type'], token['value']),
+			'autoCreate': 'true',
+			'requestUri': self.client_secret['redirect_uris'][0],
+			'sessionId': self.session_id,
+			'returnSecureToken': 'true',
+			'returnRefreshToken': 'true',
+			'returnIdpCredential': 'false',
+		}
+
+		headers = {"content-type": "application/json; charset=UTF-8"}
+		request_object = self.requests.post(request_ref, headers=headers, json=data)
+
+		raise_detailed_error(request_object)
+		self.current_user = request_object.json()
+
+		return request_object.json()
+
 	def update_profile(self, id_token, display_name=None, photo_url=None, delete_attribute=None):
 		""" Update a user's profile (display name / photo URL).
 
@@ -371,10 +471,9 @@ class Auth:
 		:param photo_url: User's new photo url.
 
 		:type delete_attribute: list[str] or None
-		:param delete_attribute: List of attributes
-
-			to delete, "DISPLAY_NAME" or "PHOTO_URL". This will nullify
-			these values.
+		:param delete_attribute: List of attributes to delete,
+			"DISPLAY_NAME" or "PHOTO_URL". This will nullify these
+			values.
 
 		:return: UserInfo and Firebase Auth Tokens.
 		:rtype: dict
@@ -389,3 +488,22 @@ class Auth:
 		raise_detailed_error(request_object)
 
 		return request_object.json()
+
+
+def _load_client_secret(secret):
+	""" Load social providers' client secret from file if file path
+	provided.
+
+	:type secret: str or dict
+	:param secret: File path to or the dict object from social client
+		secret file.
+
+	:return: social client secret
+	:rtype: dict
+	"""
+
+	if type(secret) is str:
+		with open(secret) as file:
+			secret = json.load(file)
+
+	return secret
